@@ -1,9 +1,8 @@
 """Builtin marcros for vcfstats"""
 import warnings
 from functools import partial
-from .utils import MACROS
 
-# pylint: disable=invalid-name
+from .utils import MACROS
 
 
 def categorical(func=None, alias=None, _name=None):
@@ -13,11 +12,11 @@ def categorical(func=None, alias=None, _name=None):
     funcname = func.__name__
     if funcname not in MACROS:
         MACROS[funcname] = {}
-        MACROS[funcname]['func'] = MACROS[funcname].get('func', func)
-        MACROS[funcname]['type'] = 'categorical'
+        MACROS[funcname]["func"] = MACROS[funcname].get("func", func)
+        MACROS[funcname]["type"] = "categorical"
     if _name:
         MACROS[_name] = MACROS[funcname]
-    return MACROS[funcname]['func']
+    return MACROS[funcname]["func"]
 
 
 def continuous(func=None, alias=None, _name=None):
@@ -27,11 +26,11 @@ def continuous(func=None, alias=None, _name=None):
     funcname = func.__name__
     if funcname not in MACROS:
         MACROS[funcname] = {}
-        MACROS[funcname]['func'] = MACROS[funcname].get('func', func)
-        MACROS[funcname]['type'] = 'continuous'
+        MACROS[funcname]["func"] = MACROS[funcname].get("func", func)
+        MACROS[funcname]["type"] = "continuous"
     if _name:
         MACROS[_name] = MACROS[funcname]
-    return MACROS[funcname]['func']
+    return MACROS[funcname]["func"]
 
 
 def aggregation(func=None, alias=None, _name=None):
@@ -41,11 +40,11 @@ def aggregation(func=None, alias=None, _name=None):
     funcname = func.__name__
     if funcname not in MACROS:
         MACROS[funcname] = {}
-        MACROS[funcname]['func'] = MACROS[funcname].get('func', func)
-        MACROS[funcname]['aggr'] = True
+        MACROS[funcname]["func"] = MACROS[funcname].get("func", func)
+        MACROS[funcname]["aggr"] = True
     if _name:
         MACROS[_name] = MACROS[funcname]
-    return MACROS[funcname]['func']
+    return MACROS[funcname]["func"]
 
 
 cat = categorical
@@ -65,35 +64,42 @@ def TITV(variant):
     The variant has to be an snp first."""
     if not variant.is_snp:
         return False
-    return 'transition' if variant.is_transition else 'transversion'
+    return "transition" if variant.is_transition else "transversion"
 
 
-@categorical(alias='CHROM')
+@categorical(alias="CHROM")
 def CONTIG(variant):
     """Get the config/chromosome of a variant. Alias: CHROM"""
     return variant.CHROM
 
 
-@categorical(alias='GT_TYPEs')
+@categorical(alias="GT_TYPEs")
 def GTTYPEs(variant):
     """Get the genotypes(HOM_REF,HET,HOM_ALT,UNKNOWN)
     of a variant for each sample"""
     gttypes = variant.gt_types
-    return ['HOM_REF'if gttype == 0 else \
-            'HET' if gttype == 1 else \
-            'HOM_ALT' if gttype == 2 else 'UNKNOWN' for gttype in gttypes]
+    return [
+        "HOM_REF"
+        if gttype == 0
+        else "HET"
+        if gttype == 1
+        else "HOM_ALT"
+        if gttype == 2
+        else "UNKNOWN"
+        for gttype in gttypes
+    ]
 
 
 @categorical
 def FILTER(variant):
     """Get the FILTER of a variant."""
-    return variant.FILTER or 'PASS'
+    return variant.FILTER or "PASS"
 
 
 @categorical
 def SUBST(variant):
     """Substitution of the variant, including all types of varinat"""
-    return '{}>{}'.format(variant.REF, ','.join(variant.ALT))
+    return "{}>{}".format(variant.REF, ",".join(variant.ALT))
 
 
 @continuous
@@ -114,15 +120,18 @@ def QUAL(variant):
     return variant.QUAL
 
 
-@continuous(alias='DPs')
+@continuous(alias="DPs")
 def DEPTHs(variant):
     """Get the read-depth for each sample as a numpy array."""
     try:
-        return [sum(dp) for dp in variant.format('DP')]
+        return [sum(dp) for dp in variant.format("DP")]
     except (TypeError, ValueError):
-        warnings.warn('Failed to fetch sample depth for variant: {}'.format(
-            variant).rstrip("\n"),
-                      stacklevel=0)
+        warnings.warn(
+            "Failed to fetch sample depth for variant: {}".format(
+                variant
+            ).rstrip("\n"),
+            stacklevel=0,
+        )
         return None
 
 
@@ -139,7 +148,7 @@ def AFs(variant):
 
 
 @continuous
-def _ONE(variant):  # pylint: disable=unused-argument
+def _ONE(variant):
     """Return 1 for a variant, usually used in aggregation,
     or indication of a distribution plot"""
     return 1
@@ -157,7 +166,7 @@ def SUM(entries):
     return sum(entries)
 
 
-@aggregation(alias='AVG')
+@aggregation(alias="AVG")
 def MEAN(entries):
     """Get the mean of the values"""
     if not entries:
